@@ -910,6 +910,14 @@ draw_once (GnomeBG   *bg,
 
 	pixbuf = get_pixbuf_for_size (bg, num_monitor, rect.width, rect.height);
 	if (pixbuf) {
+		GdkPixbuf *rotated;
+
+		rotated = gdk_pixbuf_apply_embedded_orientation (pixbuf);
+		if (rotated != NULL) {
+			g_object_unref (pixbuf);
+			pixbuf = rotated;
+		}
+
 		draw_image_area (bg,
 				 num_monitor,
 				 pixbuf,
@@ -1227,6 +1235,8 @@ get_filename_for_size (GnomeBG *bg, gint best_width, gint best_height)
 	}
 
         gnome_bg_slide_show_get_current_slide (show, best_width, best_height, NULL, NULL, NULL, &file, NULL);
+        g_object_unref (show);
+
         return file;
 }
 
@@ -2552,8 +2562,17 @@ create_thumbnail_for_filename (GnomeDesktopThumbnailFactory *factory,
 	else {
 		orig = gdk_pixbuf_new_from_file (filename, NULL);
 		if (orig) {
-			int orig_width = gdk_pixbuf_get_width (orig);
-			int orig_height = gdk_pixbuf_get_height (orig);
+			int orig_width, orig_height;
+			GdkPixbuf *rotated;
+
+			rotated = gdk_pixbuf_apply_embedded_orientation (orig);
+			if (rotated != NULL) {
+				g_object_unref (orig);
+				orig = rotated;
+			}
+
+			orig_width = gdk_pixbuf_get_width (orig);
+			orig_height = gdk_pixbuf_get_height (orig);
 			
 			result = pixbuf_scale_to_fit (orig, THUMBNAIL_SIZE, THUMBNAIL_SIZE);
 			
@@ -2609,6 +2628,7 @@ gboolean
 gnome_bg_changes_with_time (GnomeBG *bg)
 {
 	GnomeBGSlideShow *show;
+	gboolean ret = FALSE;
 
 	g_return_val_if_fail (bg != NULL, FALSE);
 
@@ -2616,10 +2636,12 @@ gnome_bg_changes_with_time (GnomeBG *bg)
 		return FALSE;
 
 	show = get_as_slideshow (bg, bg->filename);
-	if (show)
-		return gnome_bg_slide_show_get_num_slides (show) > 1;
+	if (show) {
+		ret = gnome_bg_slide_show_get_num_slides (show) > 1;
+		g_object_unref (show);
+	}
 
-	return FALSE;
+	return ret;
 }
 
 /**
