@@ -45,11 +45,9 @@
 
 typedef struct {
   gboolean sandbox;
-  char *thumbnailer_name;
   GArray *fd_array;
   /* Input/output file paths outside the sandbox */
   char *infile;
-  char *infile_tmp; /* the host version of /tmp/gnome-desktop-file-to-thumbnail.* */
   char *outfile;
   char *outdir; /* outdir is outfile's parent dir, if it needs to be deleted */
   /* I/O file paths inside the sandbox */
@@ -60,14 +58,14 @@ typedef struct {
 static char *
 expand_thumbnailing_elem (const char *elem,
 			  const int   size,
-			  const char *infile,
+			  const char *inuri,
 			  const char *outfile,
 			  gboolean   *got_input,
 			  gboolean   *got_output)
 {
   GString *str;
   const char *p, *last;
-  char *inuri;
+  char *localfile;
 
   str = g_string_new (NULL);
 
@@ -79,18 +77,18 @@ expand_thumbnailing_elem (const char *elem,
 
       switch (*p) {
       case 'u':
-        inuri = g_filename_to_uri (infile, NULL, NULL);
-        if (inuri)
-	  {
-	    g_string_append (str, inuri);
-	    *got_input = TRUE;
-	    g_free (inuri);
-	  }
+	g_string_append (str, inuri);
+	*got_input = TRUE;
 	p++;
 	break;
       case 'i':
-	g_string_append (str, infile);
-	*got_input = TRUE;
+	localfile = g_filename_from_uri (inuri, NULL, NULL);
+	if (localfile)
+	  {
+	    g_string_append (str, localfile);
+	    *got_input = TRUE;
+	    g_free (localfile);
+	  }
 	p++;
 	break;
       case 'o':
@@ -131,14 +129,17 @@ add_args (GPtrArray *argv_array, ...)
   va_end (args);
 }
 
-static void
-add_env (GPtrArray  *array,
-         const char *envvar)
+static char *
+get_extension (const char *path)
 {
-  if (g_getenv (envvar) != NULL)
-    add_args (array,
-              "--setenv", envvar, g_getenv (envvar),
-              NULL);
+  g_autofree char *basename = NULL;
+  char *p;
+
+  basename = g_path_get_basename (path);
+  p = strrchr (basename, '.');
+  if (p == NULL)
+    return NULL;
+  return g_strdup (p + 1);
 }
 
 #ifdef ENABLE_SECCOMP
@@ -238,10 +239,10 @@ flatpak_get_arch (void)
 }
 
 /* From https://github.com/flatpak/flatpak/blob/master/common/flatpak-run.c */
-static const guint32 seccomp_x86_64_extra_arches[] = { SCMP_ARCH_X86, 0, };
+static const uint32_t seccomp_x86_64_extra_arches[] = { SCMP_ARCH_X86, 0, };
 
 #ifdef SCMP_ARCH_AARCH64
-static const guint32 seccomp_aarch64_extra_arches[] = { SCMP_ARCH_ARM, 0 };
+static const uint32_t seccomp_aarch64_extra_arches[] = { SCMP_ARCH_ARM, 0 };
 #endif
 
 static inline void
@@ -327,17 +328,10 @@ setup_seccomp (GPtrArray  *argv_array,
     {SCMP_SYS (unshare)},
     {SCMP_SYS (mount)},
     {SCMP_SYS (pivot_root)},
-#if defined(__s390__) || defined(__s390x__) || defined(__CRIS__)
-    /* Architectures with CONFIG_CLONE_BACKWARDS2: the child stack
-     * and flags arguments are reversed so the flags come second */
-    {SCMP_SYS (clone), &SCMP_A1 (SCMP_CMP_MASKED_EQ, CLONE_NEWUSER, CLONE_NEWUSER)},
-#else
-    /* Normally the flags come first */
     {SCMP_SYS (clone), &SCMP_A0 (SCMP_CMP_MASKED_EQ, CLONE_NEWUSER, CLONE_NEWUSER)},
-#endif
 
     /* Don't allow faking input to the controlling tty (CVE-2017-5226) */
-    {SCMP_SYS (ioctl), &SCMP_A1(SCMP_CMP_MASKED_EQ, 0xFFFFFFFFu, (int)TIOCSTI)},
+    {SCMP_SYS (ioctl), &SCMP_A1(SCMP_CMP_EQ, (int)TIOCSTI)},
   };
 
   struct
@@ -367,8 +361,7 @@ setup_seccomp (GPtrArray  *argv_array,
     AF_KEY,
     AF_NETLINK + 1, /* Last gets CMP_GE, so order is important */
   };
-  guint i;
-  int r;
+  int i, r;
   int fd = -1;
   g_autofree char *fd_str = NULL;
   g_autofree char *path = NULL;
@@ -379,8 +372,8 @@ setup_seccomp (GPtrArray  *argv_array,
 
   if (arch != NULL)
     {
-      guint32 arch_id = 0;
-      const guint32 *extra_arches = NULL;
+      uint32_t arch_id = 0;
+      const uint32_t *extra_arches = NULL;
 
       if (strcmp (arch, "i386") == 0)
         {
@@ -418,6 +411,7 @@ setup_seccomp (GPtrArray  *argv_array,
 
           if (multiarch && extra_arches != NULL)
             {
+              unsigned i;
               for (i = 0; extra_arches[i] != 0; i++)
                 {
                   r = seccomp_arch_add (seccomp, extra_arches[i]);
@@ -500,31 +494,12 @@ setup_seccomp (GPtrArray  *argv_array,
 #endif
 
 #ifdef HAVE_BWRAP
-static gboolean
-path_is_usrmerged (const char *dir)
-{
-  /* does /dir point to /usr/dir? */
-  g_autofree char *target = NULL;
-  GStatBuf stat_buf_src, stat_buf_target;
-
-  if (g_stat (dir, &stat_buf_src) < 0)
-    return FALSE;
-
-  target = g_strdup_printf ("/usr/%s", dir);
-
-  if (g_stat (target, &stat_buf_target) < 0)
-    return FALSE;
-
-  return (stat_buf_src.st_dev == stat_buf_target.st_dev) &&
-         (stat_buf_src.st_ino == stat_buf_target.st_ino);
-}
-
 static gboolean
 add_bwrap (GPtrArray   *array,
 	   ScriptExec  *script)
 {
-  const char * const usrmerged_dirs[] = { "bin", "lib64", "lib", "sbin" };
-  int i;
+  char *fd_str;
+  int fd;
 
   g_return_val_if_fail (script->outdir != NULL, FALSE);
   g_return_val_if_fail (script->s_infile != NULL, FALSE);
@@ -532,61 +507,17 @@ add_bwrap (GPtrArray   *array,
   add_args (array,
 	    "bwrap",
 	    "--ro-bind", "/usr", "/usr",
-	    "--ro-bind", "/etc/ld.so.cache", "/etc/ld.so.cache",
-	    NULL);
-
-  /* These directories might be symlinks into /usr/... */
-  for (i = 0; i < G_N_ELEMENTS (usrmerged_dirs); i++)
-    {
-      g_autofree char *absolute_dir = g_strdup_printf ("/%s", usrmerged_dirs[i]);
-
-      if (!g_file_test (absolute_dir, G_FILE_TEST_EXISTS))
-        continue;
-
-      if (path_is_usrmerged (absolute_dir))
-        {
-          g_autofree char *symlink_target = g_strdup_printf ("/usr/%s", absolute_dir);
-
-          add_args (array,
-                    "--symlink", symlink_target, absolute_dir,
-                    NULL);
-        }
-      else
-        {
-          add_args (array,
-                    "--ro-bind", absolute_dir, absolute_dir,
-                    NULL);
-        }
-    }
-
-  /* fontconfig cache if necessary */
-  if (!g_str_has_prefix (FONTCONFIG_CACHE_PATH, "/usr/"))
-    add_args (array, "--ro-bind-try", FONTCONFIG_CACHE_PATH, FONTCONFIG_CACHE_PATH, NULL);
-
-  /*
-   * Used in various distributions. On those distributions, /usr is not
-   * complete without it: some files in /usr might be a symbolic link
-   * like /usr/bin/composite -> /etc/alternatives/composite ->
-   * /usr/bin/composite-im6.q16.
-   *
-   * https://manpages.debian.org/stable/dpkg/update-alternatives.1.en.html
-   * https://docs.fedoraproject.org/en-US/packaging-guidelines/Alternatives/
-   * https://en.opensuse.org/openSUSE:Packaging_Multiple_Version_guidelines
-   */
-  add_args (array, "--ro-bind-try", "/etc/alternatives", "/etc/alternatives", NULL);
-
-  add_args (array,
 	    "--proc", "/proc",
 	    "--dev", "/dev",
+	    "--symlink", "usr/lib", "/lib",
+	    "--symlink", "usr/lib64", "/lib64",
+	    "--symlink", "usr/bin", "/bin",
+	    "--symlink", "usr/sbin", "/sbin",
 	    "--chdir", "/",
-	    "--setenv", "GIO_USE_VFS", "local",
 	    "--unshare-all",
 	    "--die-with-parent",
 	    NULL);
 
-  add_env (array, "G_MESSAGES_DEBUG");
-  add_env (array, "G_MESSAGES_PREFIXED");
-
   /* Add gnome-desktop's install prefix if needed */
   if (g_strcmp0 (INSTALL_PREFIX, "") != 0 &&
       g_strcmp0 (INSTALL_PREFIX, "/usr") != 0 &&
@@ -601,14 +532,25 @@ add_bwrap (GPtrArray   *array,
   g_ptr_array_add (array, g_strdup (script->outdir));
   g_ptr_array_add (array, g_strdup ("/tmp"));
 
-  /* We make sure to also re-use the original file's original
+  /* Open the infile so we can pass the fd through bwrap,
+   * we make sure to also re-use the original file's original
    * extension in case it's useful for the thumbnailer to
    * identify the file type */
-  g_ptr_array_add (array, g_strdup ("--ro-bind"));
-  g_ptr_array_add (array, g_strdup (script->infile));
+  fd = open (script->infile, O_RDONLY | O_CLOEXEC);
+  if (fd == -1)
+    goto bail;
+  fd_str = g_strdup_printf ("%d", fd);
+  g_ptr_array_add (array, g_strdup ("--file"));
+  g_ptr_array_add (array, fd_str);
   g_ptr_array_add (array, g_strdup (script->s_infile));
 
+  g_array_append_val (script->fd_array, fd);
+
   return TRUE;
+
+bail:
+  g_ptr_array_set_size (array, 0);
+  return FALSE;
 }
 #endif /* HAVE_BWRAP */
 
@@ -622,12 +564,11 @@ expand_thumbnailing_cmd (const char  *cmd,
   g_auto(GStrv) cmd_elems = NULL;
   guint i;
   gboolean got_in, got_out;
+  g_autofree char *sandboxed_path = NULL;
 
   if (!g_shell_parse_argv (cmd, NULL, &cmd_elems, error))
     return NULL;
 
-  script->thumbnailer_name = g_strdup (cmd_elems[0]);
-
   array = g_ptr_array_new_with_free_func (g_free);
 
 #ifdef HAVE_BWRAP
@@ -702,7 +643,7 @@ static void
 child_setup (gpointer user_data)
 {
   GArray *fd_array = user_data;
-  guint i;
+  int i;
 
   /* If no fd_array was specified, don't care. */
   if (fd_array == NULL)
@@ -716,19 +657,7 @@ child_setup (gpointer user_data)
 static void
 script_exec_free (ScriptExec *exec)
 {
-  if (exec == NULL)
-    return;
-
-  g_free (exec->thumbnailer_name);
   g_free (exec->infile);
-  if (exec->infile_tmp)
-    {
-      if (g_file_test (exec->infile_tmp, G_FILE_TEST_IS_DIR))
-        g_rmdir (exec->infile_tmp);
-      else
-        g_unlink (exec->infile_tmp);
-      g_free (exec->infile_tmp);
-    }
   if (exec->outfile)
     {
       g_unlink (exec->outfile);
@@ -736,11 +665,7 @@ script_exec_free (ScriptExec *exec)
     }
   if (exec->outdir)
     {
-      if (g_rmdir (exec->outdir) < 0)
-        {
-          g_warning ("Could not remove %s, thumbnailer %s left files in directory",
-                     exec->outdir, exec->thumbnailer_name);
-        }
+      g_rmdir (exec->outdir);
       g_free (exec->outdir);
     }
   g_free (exec->s_infile);
@@ -759,8 +684,7 @@ clear_fd (gpointer data)
 }
 
 static ScriptExec *
-script_exec_new (const char  *uri,
-		 GError     **error)
+script_exec_new (const char *uri)
 {
   ScriptExec *exec;
   g_autoptr(GFile) file = NULL;
@@ -778,18 +702,13 @@ script_exec_new (const char  *uri,
 
   exec->infile = g_file_get_path (file);
   if (!exec->infile)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                   "Could not get path for URI '%s'", uri);
-      goto bail;
-    }
+    goto bail;
 
 #ifdef HAVE_BWRAP
   if (exec->sandbox)
     {
       char *tmpl;
-      const char *infile;
-      g_autofree char *basename = NULL;
+      g_autofree char *ext = NULL;
 
       exec->fd_array = g_array_new (FALSE, TRUE, sizeof (int));
       g_array_set_clear_func (exec->fd_array, clear_fd);
@@ -797,23 +716,12 @@ script_exec_new (const char  *uri,
       tmpl = g_strdup ("/tmp/gnome-desktop-thumbnailer-XXXXXX");
       exec->outdir = g_mkdtemp (tmpl);
       if (!exec->outdir)
-        {
-          g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                               "Could not create temporary sandbox directory");
-          goto bail;
-        }
+        goto bail;
       exec->outfile = g_build_filename (exec->outdir, "gnome-desktop-thumbnailer.png", NULL);
-      basename = g_file_get_basename (file);
 
-      if (basename)
-        infile = basename;
-      else
-        infile = "gnome-desktop-file-to-thumbnail";
-
-      exec->infile_tmp = g_build_filename (exec->outdir, infile, NULL);
-
-      exec->s_infile = g_build_filename ("/tmp/", infile, NULL);
-      exec->s_outfile = g_build_filename ("/tmp/", "gnome-desktop-thumbnailer.png", NULL);
+      ext = get_extension (exec->infile);
+      exec->s_infile = g_strdup_printf ("/tmp/gnome-desktop-file-to-thumbnail.%s", ext);
+      exec->s_outfile = g_strdup ("/tmp/gnome-desktop-thumbnailer.png");
     }
   else
 #endif
@@ -821,7 +729,7 @@ script_exec_new (const char  *uri,
       int fd;
       g_autofree char *tmpname = NULL;
 
-      fd = g_file_open_tmp (".gnome_desktop_thumbnail.XXXXXX", &tmpname, error);
+      fd = g_file_open_tmp (".gnome_desktop_thumbnail.XXXXXX", &tmpname, NULL);
       if (fd == -1)
         goto bail;
       close (fd);
@@ -835,22 +743,6 @@ bail:
   return NULL;
 }
 
-static void
-print_script_debug (GStrv expanded_script)
-{
-  GString *out;
-  guint i;
-
-  out = g_string_new (NULL);
-
-  for (i = 0; expanded_script[i]; i++)
-    g_string_append_printf (out, "%s ", expanded_script[i]);
-  g_string_append_printf (out, "\n");
-
-  g_debug ("About to launch script: %s", out->str);
-  g_string_free (out, TRUE);
-}
-
 GBytes *
 gnome_desktop_thumbnail_script_exec (const char  *cmd,
 				     int          size,
@@ -864,14 +756,19 @@ gnome_desktop_thumbnail_script_exec (const char  *cmd,
   GBytes *image = NULL;
   ScriptExec *exec;
 
-  exec = script_exec_new (uri, error);
-  if (!exec)
-    goto out;
+  exec = script_exec_new (uri);
   expanded_script = expand_thumbnailing_cmd (cmd, exec, size, error);
   if (expanded_script == NULL)
     goto out;
 
-  print_script_debug (expanded_script);
+#if 0
+  guint i;
+
+  g_print ("About to launch script: ");
+  for (i = 0; expanded_script[i]; i++)
+    g_print ("%s ", expanded_script[i]);
+  g_print ("\n");
+#endif
 
   ret = g_spawn_sync (NULL, expanded_script, NULL, G_SPAWN_SEARCH_PATH,
 		      child_setup, exec->fd_array, NULL, &error_out,
