@@ -55,18 +55,12 @@
  * Each is in a simple key-file format:
  * <informalexample><programlisting>
  * [Thumbnailer Entry]
- * TryExec=evince-thumbnailer
  * Exec=evince-thumbnailer -s %s %u %o
  * MimeType=application/pdf;application/x-bzpdf;application/x-gzpdf;
  * </programlisting></informalexample>
  *
  * The <filename>.thumbnailer</filename> format supports three keys:
  * <variablelist>
- * <varlistentry><term><code>TryExec</code></term><listitem><para>
- * Optional. The name of the the thumbnailer program in the current
- * <envar>$PATH</envar>. This allows the calling code to quickly check whether
- * the thumbnailer exists before trying to execute it.
- * </para></listitem></varlistentry>
  * <varlistentry><term><code>Exec</code></term><listitem><para>
  * Required. The command to execute the thumbnailer. It supports a few different
  * parameters which are replaced before calling the thumbnailer:
@@ -127,26 +121,18 @@
  */
 
 #include <config.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/time.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <dirent.h>
-#include <time.h>
-#include <math.h>
-#include <string.h>
-#include <glib.h>
-#include <stdio.h>
 
-#define GDK_PIXBUF_ENABLE_BACKEND
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <string.h>
+#include <stdlib.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include "gnome-desktop-thumbnail.h"
-#include <glib/gstdio.h>
-
-#define SECONDS_BETWEEN_STATS 10
+#include "gnome-desktop-thumbnail-script.h"
 
 static void
 thumbnailers_directory_changed (GFileMonitor                 *monitor,
@@ -188,7 +174,6 @@ typedef struct {
 
     gchar *path;
 
-    gchar  *try_exec;
     gchar  *command;
     gchar **mime_types;
 } Thumbnailer;
@@ -212,7 +197,6 @@ thumbnailer_unref (Thumbnailer *thumb)
   if (g_atomic_int_dec_and_test (&thumb->ref_count))
     {
       g_free (thumb->path);
-      g_free (thumb->try_exec);
       g_free (thumb->command);
       g_strfreev (thumb->mime_types);
 
@@ -266,8 +250,6 @@ thumbnailer_load (Thumbnailer *thumb)
       return NULL;
     }
 
-  thumb->try_exec = g_key_file_get_string (key_file, THUMBNAILER_ENTRY_GROUP, "TryExec", NULL);
-
   g_key_file_free (key_file);
 
   return thumb;
@@ -282,8 +264,6 @@ thumbnailer_reload (Thumbnailer *thumb)
   thumb->command = NULL;
   g_strfreev (thumb->mime_types);
   thumb->mime_types = NULL;
-  g_free (thumb->try_exec);
-  thumb->try_exec = NULL;
 
   return thumbnailer_load (thumb);
 }
@@ -300,45 +280,22 @@ thumbnailer_new (const gchar *path)
   return thumbnailer_load (thumb);
 }
 
-static gboolean
-thumbnailer_try_exec (Thumbnailer *thumb)
-{
-  gchar *path;
-  gboolean retval;
-
-  if (G_UNLIKELY (!thumb))
-    return FALSE;
-
-  /* TryExec is optinal, but Exec isn't, so we assume
-   * the thumbnailer can be run when TryExec is not present
-   */
-  if (!thumb->try_exec)
-    return TRUE;
-
-  path = g_find_program_in_path (thumb->try_exec);
-  retval = path != NULL;
-  g_free (path);
-
-  return retval;
-}
-
 static gpointer
 init_thumbnailers_dirs (gpointer data)
 {
   const gchar * const *data_dirs;
-  gchar **thumbs_dirs;
-  guint i, length;
+  GPtrArray *thumbs_dirs;
+  guint i;
 
   data_dirs = g_get_system_data_dirs ();
-  length = g_strv_length ((char **) data_dirs);
+  thumbs_dirs = g_ptr_array_new ();
 
-  thumbs_dirs = g_new (gchar *, length + 2);
-  thumbs_dirs[0] = g_build_filename (g_get_user_data_dir (), "thumbnailers", NULL);
-  for (i = 0; i < length; i++)
-    thumbs_dirs[i + 1] = g_build_filename (data_dirs[i], "thumbnailers", NULL);
-  thumbs_dirs[length + 1] = NULL;
+  g_ptr_array_add (thumbs_dirs, g_build_filename (g_get_user_data_dir (), "thumbnailers", NULL));
+  for (i = 0; data_dirs[i] != NULL; i++)
+    g_ptr_array_add (thumbs_dirs, g_build_filename (data_dirs[i], "thumbnailers", NULL));
+  g_ptr_array_add (thumbs_dirs, NULL);
 
-  return thumbs_dirs;
+  return g_ptr_array_free (thumbs_dirs, FALSE);
 }
 
 static const gchar * const *
@@ -667,18 +624,18 @@ static void
 gnome_desktop_thumbnail_factory_init (GnomeDesktopThumbnailFactory *factory)
 {
   GnomeDesktopThumbnailFactoryPrivate *priv;
-  
+
   factory->priv = GNOME_DESKTOP_THUMBNAIL_FACTORY_GET_PRIVATE (factory);
 
   priv = factory->priv;
 
   priv->size = GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL;
-  
+
   priv->mime_types_map = g_hash_table_new_full (g_str_hash,
                                                 g_str_equal,
                                                 (GDestroyNotify)g_free,
                                                 (GDestroyNotify)thumbnailer_unref);
-  
+
   g_mutex_init (&priv->lock);
 
   priv->settings = g_settings_new ("org.gnome.desktop.thumbnailers");
@@ -701,7 +658,7 @@ gnome_desktop_thumbnail_factory_finalize (GObject *object)
 {
   GnomeDesktopThumbnailFactory *factory;
   GnomeDesktopThumbnailFactoryPrivate *priv;
-  
+
   factory = GNOME_DESKTOP_THUMBNAIL_FACTORY (object);
 
   priv = factory->priv;
@@ -745,7 +702,7 @@ gnome_desktop_thumbnail_factory_class_init (GnomeDesktopThumbnailFactoryClass *c
   GObjectClass *gobject_class;
 
   gobject_class = G_OBJECT_CLASS (class);
-	
+
   gobject_class->finalize = gnome_desktop_thumbnail_factory_finalize;
 
   g_type_class_add_private (class, sizeof (GnomeDesktopThumbnailFactoryPrivate));
@@ -758,7 +715,7 @@ gnome_desktop_thumbnail_factory_class_init (GnomeDesktopThumbnailFactoryClass *c
  * Creates a new #GnomeDesktopThumbnailFactory.
  *
  * This function must be called on the main thread.
- * 
+ *
  * Return value: a new #GnomeDesktopThumbnailFactory
  *
  * Since: 2.2
@@ -767,11 +724,11 @@ GnomeDesktopThumbnailFactory *
 gnome_desktop_thumbnail_factory_new (GnomeDesktopThumbnailSize size)
 {
   GnomeDesktopThumbnailFactory *factory;
-  
+
   factory = g_object_new (GNOME_DESKTOP_TYPE_THUMBNAIL_FACTORY, NULL);
-  
+
   factory->priv->size = size;
-  
+
   return factory;
 }
 
@@ -934,7 +891,7 @@ gnome_desktop_thumbnail_factory_has_valid_failed_thumbnail (GnomeDesktopThumbnai
  * @mime_type: the mime type of the file
  * @mtime: the mtime of the file
  *
- * Returns TRUE if this GnomeIconFactory can (at least try) to thumbnail
+ * Returns TRUE if this GnomeDesktopThumbnailFactory can (at least try) to thumbnail
  * this file. Thumbnails or files with failed thumbnails won't be thumbnailed.
  *
  * Usage of this function is threadsafe.
@@ -956,18 +913,13 @@ gnome_desktop_thumbnail_factory_can_thumbnail (GnomeDesktopThumbnailFactory *fac
       strncmp (uri, "file:/", 6) == 0 &&
       strstr (uri, "/thumbnails/") != NULL)
     return FALSE;
-  
+
   if (!mime_type)
     return FALSE;
 
   g_mutex_lock (&factory->priv->lock);
   if (!gnome_desktop_thumbnail_factory_is_disabled (factory, mime_type))
-    {
-      Thumbnailer *thumb;
-
-      thumb = g_hash_table_lookup (factory->priv->mime_types_map, mime_type);
-      have_script = thumbnailer_try_exec (thumb);
-    }
+    have_script = TRUE;
   g_mutex_unlock (&factory->priv->lock);
 
   if (have_script)
@@ -976,79 +928,10 @@ gnome_desktop_thumbnail_factory_can_thumbnail (GnomeDesktopThumbnailFactory *fac
                                                                           uri,
                                                                           mtime);
     }
-  
+
   return FALSE;
 }
 
-static char *
-expand_thumbnailing_script (const char *script,
-			    const int   size, 
-			    const char *inuri,
-			    const char *outfile)
-{
-  GString *str;
-  const char *p, *last;
-  char *localfile, *quoted;
-  gboolean got_in;
-
-  str = g_string_new (NULL);
-  
-  got_in = FALSE;
-  last = script;
-  while ((p = strchr (last, '%')) != NULL)
-    {
-      g_string_append_len (str, last, p - last);
-      p++;
-
-      switch (*p) {
-      case 'u':
-	quoted = g_shell_quote (inuri);
-	g_string_append (str, quoted);
-	g_free (quoted);
-	got_in = TRUE;
-	p++;
-	break;
-      case 'i':
-	localfile = g_filename_from_uri (inuri, NULL, NULL);
-	if (localfile)
-	  {
-	    quoted = g_shell_quote (localfile);
-	    g_string_append (str, quoted);
-	    got_in = TRUE;
-	    g_free (quoted);
-	    g_free (localfile);
-	  }
-	p++;
-	break;
-      case 'o':
-	quoted = g_shell_quote (outfile);
-	g_string_append (str, quoted);
-	g_free (quoted);
-	p++;
-	break;
-      case 's':
-	g_string_append_printf (str, "%d", size);
-	p++;
-	break;
-      case '%':
-	g_string_append_c (str, '%');
-	p++;
-	break;
-      case 0:
-      default:
-	break;
-      }
-      last = p;
-    }
-  g_string_append (str, last);
-
-  if (got_in)
-    return g_string_free (str, FALSE);
-
-  g_string_free (str, TRUE);
-  return NULL;
-}
-
 static GdkPixbuf *
 get_preview_thumbnail (const char *uri,
                        int         size)
@@ -1105,6 +988,30 @@ get_preview_thumbnail (const char *uri,
     return pixbuf;
 }
 
+static GdkPixbuf *
+pixbuf_new_from_bytes (GBytes  *bytes,
+		       GError **error)
+{
+  g_autoptr(GdkPixbufLoader) loader = NULL;
+
+  loader = gdk_pixbuf_loader_new_with_mime_type ("image/png", error);
+  if (!loader)
+    return NULL;
+
+  if (!gdk_pixbuf_loader_write (loader,
+				g_bytes_get_data (bytes, NULL),
+				g_bytes_get_size (bytes),
+				error))
+    {
+      return NULL;
+    }
+
+  if (!gdk_pixbuf_loader_close (loader, error))
+    return NULL;
+
+  return g_object_ref (gdk_pixbuf_loader_get_pixbuf (loader));
+}
+
 /**
  * gnome_desktop_thumbnail_factory_generate_thumbnail:
  * @factory: a #GnomeDesktopThumbnailFactory
@@ -1126,16 +1033,14 @@ gnome_desktop_thumbnail_factory_generate_thumbnail (GnomeDesktopThumbnailFactory
 						    const char            *mime_type)
 {
   GdkPixbuf *pixbuf;
-  char *script, *expanded_script;
+  char *script;
   int size;
-  int exit_status;
-  char *tmpname;
 
   g_return_val_if_fail (uri != NULL, NULL);
   g_return_val_if_fail (mime_type != NULL, NULL);
 
   /* Doesn't access any volatile fields in factory, so it's threadsafe */
-  
+
   size = 128;
   if (factory->priv->size == GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE)
     size = 256;
@@ -1157,34 +1062,21 @@ gnome_desktop_thumbnail_factory_generate_thumbnail (GnomeDesktopThumbnailFactory
         script = g_strdup (thumb->command);
     }
   g_mutex_unlock (&factory->priv->lock);
-  
+
   if (script)
     {
-      int fd;
+      GBytes *data;
 
-      fd = g_file_open_tmp (".gnome_desktop_thumbnail.XXXXXX", &tmpname, NULL);
-
-      if (fd != -1)
-	{
-	  close (fd);
-
-	  expanded_script = expand_thumbnailing_script (script, size, uri, tmpname);
-	  if (expanded_script != NULL &&
-	      g_spawn_command_line_sync (expanded_script,
-					 NULL, NULL, &exit_status, NULL) &&
-	      exit_status == 0)
-	    {
-	      pixbuf = gdk_pixbuf_new_from_file (tmpname, NULL);
-	    }
-
-	  g_free (expanded_script);
-	  g_unlink (tmpname);
-	  g_free (tmpname);
-	}
-
-      g_free (script);
+      data = gnome_desktop_thumbnail_script_exec (script, size, uri, NULL);
+      if (data)
+        {
+          pixbuf = pixbuf_new_from_bytes (data, NULL);
+          g_bytes_unref (data);
+        }
     }
 
+  g_free (script);
+
   return pixbuf;
 }
 
@@ -1386,12 +1278,10 @@ gnome_desktop_thumbnail_has_uri (GdkPixbuf          *pixbuf,
 				 const char         *uri)
 {
   const char *thumb_uri;
-  
-  thumb_uri = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::URI");
-  if (!thumb_uri)
-    return FALSE;
 
-  return strcmp (uri, thumb_uri) == 0;
+  thumb_uri = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::URI");
+
+  return (g_strcmp0 (uri, thumb_uri) == 0);
 }
 
 /**
@@ -1414,19 +1304,17 @@ gnome_desktop_thumbnail_is_valid (GdkPixbuf          *pixbuf,
 {
   const char *thumb_uri, *thumb_mtime_str;
   time_t thumb_mtime;
-  
+
   thumb_uri = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::URI");
-  if (!thumb_uri)
+  if (g_strcmp0 (uri, thumb_uri) != 0)
     return FALSE;
-  if (strcmp (uri, thumb_uri) != 0)
-    return FALSE;
-  
+
   thumb_mtime_str = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::MTime");
   if (!thumb_mtime_str)
     return FALSE;
   thumb_mtime = atol (thumb_mtime_str);
   if (mtime != thumb_mtime)
     return FALSE;
-  
+
   return TRUE;
 }
